/*----------------------------------------------------------
   Sends mail through the local SMTP server.

           Author:       Peter Moylan (peter@ee.newcastle.edu.au)
           Started:      12 November 1999
           Last revised: 17 December 1999

   Usage:
           send -f sender@localdomain -t recipient@remotedomain filename

           where the argument after the '-f' specifies the sender;
           the argument after the -t specifies the address to which
           the mail is addressed; and filename is the file that is
           to be sent.  (This file must contain both the header and
           the body of the message.)  The arguments may be given in
           any order.  If the -f and/or -t arguments are missing
           then the addresses are taken from the From: and/or To:
           lines in the header of the message.

   Limitation:
           The present version can't handle more than one recipient.

------------------------------------------------------------*/

/****************************************************************/
/*                       MAIN PROGRAM                           */
/****************************************************************/

CALL RxFuncAdd SysLoadFuncs, rexxutil, sysloadfuncs
CALL SysLoadFuncs
CALL RxFuncAdd "SockLoadFuncs","rxSock","SockLoadFuncs"
CALL SockLoadFuncs nul
RB = ""                                       /* Receive buffer */
Globals = "OurSocket RB ReturnPath ForwardPath FileName"

PARSE ARG userargs
CALL ParseArgs userargs
IF (ReturnPath = "") | (ForwardPath = "") THEN CALL CheckHeader
ForwardPath = CleanUpAddress(ForwardPath)
ReturnPath = CleanUpAddress(ReturnPath)

IF ConnectToServer() THEN
    DO
        IF ExecCmd("MAIL FROM:<"ReturnPath">") THEN
           DO
              IF ExecCmd("RCPT TO:<"ForwardPath">") THEN
                 DO
                    IF ExecCmd("DATA") THEN
                       DO
                           CALL SendFile
                       END
                    ELSE SAY "DATA command failed"
                 END
              ELSE SAY "Server refused mail to "ForwardPath
           END
        ELSE SAY "Server won't accept mail from "ReturnPath
    END

IF OurSocket \= -1 THEN
    DO
        CALL ExecCmd "QUIT"
        CALL SockSoClose(OurSocket)
    END

EXIT

/****************************************************************/
/*              PARSING THE COMMAND-LINE ARGUMENTS              */
/****************************************************************/

ParseArgs: PROCEDURE expose (Globals)
   ReturnPath = ""
   ForwardPath = ""
   FileName = ""
   parse arg head tail
   DO WHILE head \= ""
       IF head = "-f" THEN
              PARSE VAR tail ReturnPath head tail
       ELSE IF head = "-t" THEN
              PARSE VAR tail ForwardPath head tail
       ELSE
           DO
              FileName = head
              PARSE VAR tail head tail
           END
   END
   RETURN

/****************************************************************/
/*                SENDING ONE FILE TO THE SERVER                */
/****************************************************************/

SendFile: PROCEDURE expose (Globals)

   linecount = 0
   DO WHILE LINES(FileName) > 0
       line = LineIn(FileName)
       IF LEFT(line,1) = '.' THEN line = '.'||line
       CALL SendLine line
       linecount = linecount + 1
       IF linecount//10 = 0 THEN CALL CHAROUT ,'.'
   END
   CALL ExecCmd "."
   SAY;SAY linecount" lines sent"
   RETURN

/****************************************************************/
/*      GETTING "To:" AND/OR "From:" LINES FROM THE HEADER      */
/****************************************************************/

CheckHeader: PROCEDURE EXPOSE (Globals)

    InHeader = 1

    DO WHILE InHeader & ((ForwardPath = "") | (ReturnPath = ""))

        /* Each time around this loop we look at one line of   */
        /* the header.                                         */

        line = LineIn(FileName)
        IF line = "" THEN InHeader = 0
        ELSE IF (LEFT(line,1) = ' ') THEN
            DO
               /* Continuation line, do nothing */
            END
        ELSE IF POS(':', line) = 0 THEN InHeader = 0
        ELSE
            DO
               PARSE VALUE line WITH keyword ':' remainder
               IF (keyword = "From") & (ReturnPath = "") THEN
                   ReturnPath = remainder
               ELSE IF (keyword = "To") & (ForwardPath = "") THEN
                   ForwardPath = remainder
            END
    END
    CALL stream FileName, 'C', 'CLOSE'
    RETURN

/****************************************************************/
/*                 TIDYING UP AN E-MAIL ADDRESS                 */
/****************************************************************/

CleanupAddress: PROCEDURE

    /* The argument to this procedure is the "To:" line from    */
    /* the message header.  It has the form user@domain, plus   */
    /* possibly some other detail.  We return the user@domain,  */
    /* stripping off the rest.                                  */

    PARSE ARG line
    line = STRIP(line)
    IF LEFT(line,1) = '"' THEN
        PARSE VAR line WITH '"' rubbish '"' line
    IF RIGHT(line,1) = ')' THEN
        PARSE VAR line WITH line '(' rubbish ')'
    line = STRIP(line)
    IF LEFT(line,1) = '<' THEN
        PARSE VAR line WITH '<' line '>' rubbish
    RETURN line

/****************************************************************/
/*                 CONNECTING TO THE SERVER                     */
/****************************************************************/

ConnectToServer: procedure expose (Globals)

    /* This procedure opens a connection to the local SMTP server. */

    OurSocket = SockSocket("AF_INET", "SOCK_STREAM", "IPPROTO_TCP")
    IF OurSocket = -1 THEN
      DO
        say "Can't create socket"
        RETURN 0
      END

    target.!family = 'AF_INET'
    target.!port = 25
    target.!addr = 127.0.0.1
    DROP host.
    IF SockConnect(OurSocket,"target.!") = -1 THEN
      DO
        say "Failed to connect"
        RETURN 0
      END
    IF \OKReply() THEN
      DO
        say "Server rejected us"
        RETURN 0
      END
    IF \ExecCmd("HELO localhost") THEN
      DO
        say "HELO command rejected"
        RETURN 0
      END
    RETURN 1

/****************************************************************/

SendLine: PROCEDURE expose (Globals)

    /* Sends a single line to the server */

    PARSE ARG str
    RETURN (SockSend(OurSocket, str||'0D0A'X) \= -1)

/****************************************************************/

ExecCmd: PROCEDURE expose (Globals)

    /* Like SendLine, but also checks for a positive reply. */

    PARSE ARG str
    RETURN (SockSend(OurSocket, str||'0D0A'X) \= -1) & OKReply()

/****************************************************************/

OKReply: PROCEDURE expose (Globals)

    /* Checks that server returned an "OK" response */

    ch = LEFT(ReceiveLine(),1)
    RETURN (ch = '1') | (ch = '2') | (ch = '3')

/****************************************************************/
/*              RECEIVING ONE LINE FROM THE SERVER              */
/*                                                              */
/*  Note: the global variable RB is the receive buffer; it      */
/*  holds input that has not yet been processed.                */
/****************************************************************/

ReceiveLine: PROCEDURE expose (Globals)
    line = ''
    DO FOREVER
      len = LENGTH(RB)
      IF len = 0 THEN
        DO
          NullResponseCount = 0
          DO WHILE len = 0
            len = SockRecv(OurSocket, 'RB', 256)
            IF len = 0 THEN
              DO
                NullResponseCount = NullResponseCount+1
                IF NullResponseCount > 20 THEN len = -1
              END
          END /*do-while*/
          IF len = -1 THEN RETURN ''
        END /* if len=0 */
      j0 = POS('0A'X, RB)
      IF j0 = 0 THEN
        DO
          line = STRIP(line||RB, 'T', '0D'X)
          RB = ''
        END
      ELSE
        DO
          line = STRIP(line||LEFT(RB,j0-1), 'T', '0D'X)
          RB = RIGHT(RB,len-j0)
          RETURN line
        END
    END /* do forever */

/****************************************************************/

